﻿using NevaevaLibrary.Exceptions;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace NevaevaLibrary
{
    public partial class ListBoxControl : UserControl
    {
        public ListBoxControl()
        {
            InitializeComponent();
        }

        public string? template;
        public char? fromChar;
        public char? toChar;

        public void setItems<T>(List<T> items)
            where T : class
        {
            if (template == null || !fromChar.HasValue || !toChar.HasValue) throw new ArgumentException("Не хватает данных!");
            listBox.Items.Clear();
            var type = typeof(T);
            var fields = type.GetFields();
            foreach (T item in items)
            {
                string result = template;
                foreach (var field in fields)
                {
                    string search = fromChar.Value + field.Name + toChar.Value;
                    result = result.Replace(search, field.GetValue(item).ToString());
                }
                listBox.Items.Add(result);
            }
        }

        public T? getSelectedItem<T>()
        {
            if (template == null || !fromChar.HasValue || !toChar.HasValue || listBox.SelectedIndex == -1) throw new ArgumentException("Не хватает данных!");
            var type = typeof(T);
            var fields = type.GetFields();

            var item = Activator.CreateInstance(type);

            string[] wordsTemplate = template.Split(' ');
            string[] words = listBox.SelectedItem.ToString().Split(' ');
            for (int i = 0; i < wordsTemplate.Length; i++)
            {
                string word = wordsTemplate[i];
                if (word.StartsWith(fromChar.Value))
                {
                    string fieldName = word.Substring(1, word.Length - 2);
                    var field = fields.FirstOrDefault(x => x.Name == fieldName);
                    if (field == null) continue;
                    int indexBefore = i > 0 ? Array.IndexOf(words, wordsTemplate[i-1]) : -1;
                    int indexAfter = i < wordsTemplate.Length - 1 ? Array.IndexOf(words, wordsTemplate[i+1]) : wordsTemplate.Length - 1;
                    var fieldValue = String.Join(' ', words.Skip(indexBefore + 1).Take(indexAfter - indexBefore - 1).ToArray());
                    field.SetValue(item, Convert.ChangeType(fieldValue, field.FieldType));
                }
            }

            return item != null ? (T)item : default(T);
        }

        public int SelectedIndex
        {
            get
            {
                return listBox.SelectedIndex;
            }
            set
            {
                listBox.SelectedIndex = value;
            }
        }

        public void setParams(string template, char fromChar, char toChar)
        {
            this.template = template;
            this.fromChar = fromChar;
            this.toChar = toChar;
        }
    }
}